@@ -11,6 +11,7 @@
 #include "ioctl_wrappers.h"
 #include "gpgpu_shader.h"
 #include "gpu_cmds.h"
+#include "xe/xe_query.h"
 
 struct label_entry {
 	uint32_t id;
@@ -116,6 +117,7 @@ __xelp_gpgpu_execfunc(struct intel_bb *ibb,
 							      4 * shdr->size);
 	idd = intel_bb_ptr_get(ibb, interface_descriptor);
 	idd->desc2.illegal_opcode_exception_enable = shdr->illegal_opcode_exception_enable;
+	idd->desc6.num_threads_in_tg = shdr->num_threads_in_tg;
 
 	if (sip && sip->size)
 		sip_offset = fill_sip(ibb, sip->instr, 4 * sip->size);
@@ -130,7 +132,7 @@ __xelp_gpgpu_execfunc(struct intel_bb *ibb,
 
 	gen9_emit_state_base_address(ibb);
 
-	xelp_emit_vfe_state(ibb, THREADS, GEN8_GPGPU_URB_ENTRIES,
+	xelp_emit_vfe_state(ibb, xe_query_eu_thread_count(ibb->fd, 0), GEN8_GPGPU_URB_ENTRIES,
 			    GPGPU_URB_SIZE, GPGPU_CURBE_SIZE, true);
 
 	gen7_emit_interface_descriptor_load(ibb, interface_descriptor);
@@ -178,6 +180,7 @@ __xehp_gpgpu_execfunc(struct intel_bb *ibb,
 	xehp_fill_interface_descriptor(ibb, target, shdr->instr,
 				       4 * shdr->size, &idd);
 	idd.desc2.illegal_opcode_exception_enable = shdr->illegal_opcode_exception_enable;
+	idd.desc5.num_threads_in_tg = shdr->num_threads_in_tg;
 
 	if (shdr->vrt != VRT_DISABLED)
 		idd.desc2.registers_per_thread = shdr->vrt;
@@ -195,7 +198,7 @@ __xehp_gpgpu_execfunc(struct intel_bb *ibb,
 	xehp_emit_state_base_address(ibb);
 	xehp_emit_state_compute_mode(ibb, shdr->vrt != VRT_DISABLED);
 	xehp_emit_state_binding_table_pool_alloc(ibb);
-	xehp_emit_cfe_state(ibb, THREADS);
+	xehp_emit_cfe_state(ibb, xe_query_eu_thread_count(ibb->fd, 0));
 
 	if (sip_offset)
 		emit_sip(ibb, sip_offset);
@@ -279,6 +282,10 @@ struct gpgpu_shader *gpgpu_shader_create(int fd)
 	shdr->max_size = 16 * 4;
 	shdr->code = malloc(4 * shdr->max_size);
 	shdr->labels = igt_map_create(igt_map_hash_32, igt_map_equal_32);
+	shdr->num_threads_in_tg = 1;
+	shdr->large_grf_mode = false;
+	shdr->simd_size = 16;  /* Default SIMD size */
+	shdr->hw_local_id_generation = false;
 	shdr->vrt = VRT_DISABLED;
 	igt_assert(shdr->code);
 
@@ -330,6 +337,136 @@ void gpgpu_shader_set_vrt(struct gpgpu_shader *shdr, enum gpgpu_shader_vrt_modes
 	shdr->vrt = vrt;
 }
 
+struct max_threads_config {
+	bool large_grf_mode;
+	uint32_t simd_size;
+	bool hw_local_id_generation;
+	uint32_t max_threads;
+};
+
+static uint32_t compute_max_threads_in_tg_xe2(bool large_grf_mode,
+					      uint32_t simd_size,
+					      bool hw_local_id_generation)
+{
+	/* BSpec: 56590 */
+	static const struct max_threads_config configs[] = {
+		/* large_grf_mode, simd_size, hw_local_id_gen, max_threads */
+		{ true,  16, false, 32 },
+		{ true,  16, true,  32 },
+		{ true,  32, false, 32 },
+		{ true,  32, true,  32 },
+		{ false, 16, false, 64 },
+		{ false, 16, true,  64 },
+		{ false, 32, false, 64 },
+		{ false, 32, true,  32 },
+	};
+
+	for (int i = 0; i < ARRAY_SIZE(configs); i++) {
+		if (configs[i].large_grf_mode == large_grf_mode &&
+		    configs[i].simd_size == simd_size &&
+		    configs[i].hw_local_id_generation == hw_local_id_generation)
+			return configs[i].max_threads;
+	}
+
+	igt_warn("Unsupported configuration: large_grf=%d, simd=%d, hw_local_id=%d\n",
+		 large_grf_mode, simd_size, hw_local_id_generation);
+	return 1;
+}
+
+struct vrt_max_threads_config {
+	enum gpgpu_shader_vrt_modes register_size;
+	uint32_t simd_size;
+	bool hw_local_id_generation;
+	uint32_t max_threads;
+};
+
+static uint32_t compute_max_threads_in_tg_xe3(enum gpgpu_shader_vrt_modes register_size,
+					      uint32_t simd_size,
+					      bool hw_local_id_generation)
+{
+	/* BSpec: 56590 */
+	static const struct vrt_max_threads_config configs[] = {
+		/* register_size, simd_size, hw_local_id_gen, max_threads */
+		/* Reg-size <= 128: SIMD16 always allows 64 threads */
+		{ VRT_32,  16, false, 64 },
+		{ VRT_32,  16, true,  64 },
+		{ VRT_64,  16, false, 64 },
+		{ VRT_64,  16, true,  64 },
+		{ VRT_96,  16, false, 64 },
+		{ VRT_96,  16, true,  64 },
+		{ VRT_128, 16, false, 64 },
+		{ VRT_128, 16, true,  64 },
+		/* Reg-size <= 128: */
+		{ VRT_32,  32, false, 64 },
+		{ VRT_32,  32, true,  32 },
+		{ VRT_64,  32, false, 64 },
+		{ VRT_64,  32, true,  32 },
+		{ VRT_96,  32, false, 64 },
+		{ VRT_96,  32, true,  32 },
+		{ VRT_128, 32, false, 64 },
+		{ VRT_128, 32, true,  32 },
+		/* Reg-size 160 */
+		{ VRT_160, 16, false, 48 },
+		{ VRT_160, 16, true,  48 },
+		{ VRT_160, 32, false, 48 },
+		{ VRT_160, 32, true,  32 },
+		/* Reg-size 192 */
+		{ VRT_192, 16, false, 40 },
+		{ VRT_192, 16, true,  40 },
+		{ VRT_192, 32, false, 40 },
+		{ VRT_192, 32, true,  32 },
+		/* Reg-size 256 */
+		{ VRT_256, 16, false, 32 },
+		{ VRT_256, 16, true,  32 },
+		{ VRT_256, 32, false, 32 },
+		{ VRT_256, 32, true,  32 },
+	};
+
+	for (int i = 0; i < ARRAY_SIZE(configs); i++) {
+		if (configs[i].register_size == register_size &&
+		    configs[i].simd_size == simd_size &&
+		    configs[i].hw_local_id_generation == hw_local_id_generation)
+			return configs[i].max_threads;
+	}
+
+	igt_warn("Unsupported configuration: register_size=%d, simd=%d, hw_local_id=%d\n",
+		 register_size, simd_size, hw_local_id_generation);
+	return 1;
+}
+
+/**
+ * gpgpu__shader_get_max_threads_in_tg:
+ * @shdr: shader to query
+ *
+ * Returns the maximum number of threads in thread group for the given shader
+ * based on its current configuration (VRT mode, SIMD size, etc.) and Xe version.
+ *
+ * Returns: maximum number of threads in thread group
+ */
+uint32_t gpgpu_shader__get_max_threads_in_tg(struct gpgpu_shader *shdr)
+{
+	enum gpgpu_shader_vrt_modes register_size = shdr->vrt;
+
+	/* Not implemented for Xe platforms  */
+	if (shdr->gen_ver < 2000)
+		return 1;
+
+	/* Xe2 platforms */
+	if (shdr->gen_ver < 3000) {
+		return compute_max_threads_in_tg_xe2(shdr->large_grf_mode,
+						     shdr->simd_size,
+						     shdr->hw_local_id_generation);
+	}
+
+	/* Xe3 platforms */
+	if (shdr->vrt == VRT_DISABLED) {
+		/* BSpec: 60258 */
+		register_size = shdr->large_grf_mode ? VRT_256 : VRT_128;
+	}
+	return compute_max_threads_in_tg_xe3(register_size, shdr->simd_size,
+					     shdr->hw_local_id_generation);
+}
+
 /**
  * gpgpu_shader__breakpoint_on:
  * @shdr: shader to create breakpoint in
