@@ -1290,9 +1290,9 @@ uint64_t emit_blt_ctrl_surf_copy(int fd,
 	size_t data_sz;
 	uint64_t dst_offset, src_offset, bb_offset, alignment;
 	uint32_t bbe = MI_BATCH_BUFFER_END;
-	uint32_t *bb;
 	uint32_t ccs_per_page, max_blocks, src_step, dst_step;
 	int32_t left_blocks;
+	uint8_t *bb;
 
 	igt_assert_f(ahnd, "ctrl-surf-copy supports softpin only\n");
 	igt_assert_f(surf, "ctrl-surf-copy requires data to do ctrl-surf-copy blit\n");
@@ -1408,9 +1408,9 @@ uint64_t emit_blt_ctrl_surf_copy(int fd,
 	}
 
 	if (emit_bbe) {
-		igt_assert(bb_pos + sizeof(uint32_t) < surf->bb.size);
+		igt_assert(bb_pos + sizeof(bbe) < surf->bb.size);
 		memcpy(bb + bb_pos, &bbe, sizeof(bbe));
-		bb_pos += sizeof(uint32_t);
+		bb_pos += sizeof(bbe);
 	}
 
 	munmap(bb, surf->bb.size);
@@ -1629,6 +1629,60 @@ static void dump_bb_fast_cmd(struct gen12_fast_copy_data *data)
 		 cmd[9], data->dw09.src_address_hi);
 }
 
+struct xe_flush_dw_cmd {
+	struct {
+		uint32_t length:	BITRANGE(0, 5);
+		uint32_t rsvd0:		BITRANGE(6, 7);
+		uint32_t notify_en: 	BITRANGE(8, 8);
+		uint32_t flush_llc:	BITRANGE(9, 9);
+		uint32_t rsvd1:		BITRANGE(10, 13);
+		uint32_t postsync:	BITRANGE(14, 15);
+		uint32_t flush_ccs:	BITRANGE(16, 16);
+		uint32_t rsvd2:		BITRANGE(17, 17);
+		uint32_t tlb_inval:	BITRANGE(18, 18);
+		uint32_t rsvd3:		BITRANGE(19, 20);
+		uint32_t store_index:	BITRANGE(21, 21);
+		uint32_t rsvd4:		BITRANGE(22, 22);
+		uint32_t opcode:	BITRANGE(23, 28);
+		uint32_t client:	BITRANGE(29, 31);
+	} dw00;
+
+	uint32_t dw01;
+	uint32_t dw02;
+	uint32_t dw03;
+	uint32_t dw04;
+};
+
+/**
+ * emit_xe_flush_dw:
+ * @fd: drm fd
+ * @blt: blitter data which provides the BB pointer
+ * @bb_pos: position at which to insert the flush command
+ *
+ * Function emits a flush instruction to the BB
+ *
+ * Returns:
+ * Next write position in batch.
+ */
+uint64_t emit_xe_flush_dw(int fd, const struct blt_copy_data *blt,
+			  uint64_t bb_pos)
+{
+	uint8_t *bb;
+	struct xe_flush_dw_cmd data = {};
+
+	igt_assert(bb_pos + sizeof(data) < blt->bb.size);
+
+	bb = bo_map(fd, blt->bb.handle, blt->bb.size, blt->driver);
+	data.dw00.opcode = 0x26;
+	data.dw00.length = 3;
+	memcpy(bb + bb_pos, &data, sizeof(data));
+	bb_pos += sizeof(data);
+
+	munmap(bb, blt->bb.size);
+
+	return bb_pos;
+}
+
 /**
  * emit_blt_fast_copy:
  * @fd: drm fd
@@ -1655,7 +1709,7 @@ uint64_t emit_blt_fast_copy(int fd,
 	struct gen12_fast_copy_data data = {};
 	uint64_t dst_offset, src_offset, bb_offset;
 	uint32_t bbe = MI_BATCH_BUFFER_END;
-	uint32_t *bb;
+	uint8_t *bb;
 
 	data.dw00.client = 0x2;
 	data.dw00.opcode = 0x42;
@@ -1717,9 +1771,9 @@ uint64_t emit_blt_fast_copy(int fd,
 	bb_pos += sizeof(data);
 
 	if (emit_bbe) {
-		igt_assert(bb_pos + sizeof(uint32_t) < blt->bb.size);
+		igt_assert(bb_pos + sizeof(bbe) < blt->bb.size);
 		memcpy(bb + bb_pos, &bbe, sizeof(bbe));
-		bb_pos += sizeof(uint32_t);
+		bb_pos += sizeof(bbe);
 	}
 
 	if (blt->print_bb) {
@@ -1800,17 +1854,32 @@ int blt_fast_copy(int fd,
 
 struct xe_mem_copy_data {
 	struct {
-		uint32_t length:			BITRANGE(0, 7);
-		uint32_t compression_format:	BITRANGE(8, 12);
-		uint32_t compression_enable:	BITRANGE(13, 13);
-		uint32_t rsvd0:			BITRANGE(14, 14);
-		uint32_t dst_compressible:	BITRANGE(15, 15);
-		uint32_t src_compressible:	BITRANGE(16, 16);
-		uint32_t copy_type:		BITRANGE(17, 18);
-		uint32_t mode:			BITRANGE(19, 19);
-		uint32_t rsvd1:			BITRANGE(20, 21);
-		uint32_t opcode:			BITRANGE(22, 28);
-		uint32_t client:			BITRANGE(29, 31);
+		union {
+			struct {
+				uint32_t length:		BITRANGE(0, 7);
+				uint32_t rsvd0:			BITRANGE(8, 8);
+				uint32_t compression_format:	BITRANGE(9, 12);
+				uint32_t rsvd1:			BITRANGE(13, 16);
+				uint32_t copy_type:		BITRANGE(17, 18);
+				uint32_t mode:			BITRANGE(19, 19);
+				uint32_t rsvd2:			BITRANGE(20, 21);
+				uint32_t opcode:		BITRANGE(22, 28);
+				uint32_t client:		BITRANGE(29, 31);
+			} xe2;
+			struct {
+				uint32_t length:		BITRANGE(0, 7);
+				uint32_t compression_format:	BITRANGE(8, 12);
+				uint32_t compression_enable:	BITRANGE(13, 13);
+				uint32_t rsvd0:			BITRANGE(14, 14);
+				uint32_t dst_compressible:	BITRANGE(15, 15);
+				uint32_t src_compressible:	BITRANGE(16, 16);
+				uint32_t copy_type:		BITRANGE(17, 18);
+				uint32_t mode:			BITRANGE(19, 19);
+				uint32_t rsvd1:			BITRANGE(20, 21);
+				uint32_t opcode:		BITRANGE(22, 28);
+				uint32_t client:		BITRANGE(29, 31);
+			} gen12;
+		};
 	} dw00;
 
 	struct {
@@ -1859,9 +1928,22 @@ struct xe_mem_copy_data {
 	} dw08;
 
 	struct {
-		uint32_t dst_mocs:		BITRANGE(0, 6);
-		uint32_t rsvd0:			BITRANGE(7, 24);
-		uint32_t src_mocs:		BITRANGE(25, 31);
+		union {
+			struct {
+				uint32_t dst_encrypt:		BITRANGE(0, 0);
+				uint32_t rsvd0:			BITRANGE(1, 2);
+				uint32_t dst_mocs:		BITRANGE(3, 6);
+				uint32_t rsvd1:			BITRANGE(7, 24);
+				uint32_t src_encrypt:		BITRANGE(25, 25);
+				uint32_t rsvd2:			BITRANGE(26, 27);
+				uint32_t src_mocs:		BITRANGE(28, 31);
+			} xe2;
+			struct {
+				uint32_t dst_mocs:		BITRANGE(0, 6);
+				uint32_t rsvd0:			BITRANGE(7, 24);
+				uint32_t src_mocs:		BITRANGE(25, 31);
+			} gen12;
+		};
 	} dw09;
 };
 
@@ -1887,18 +1969,32 @@ void blt_mem_copy_init(int fd, struct blt_mem_copy_data *mem,
 	mem->copy_type = copy_type;
 }
 
-static void dump_bb_mem_copy_cmd(struct xe_mem_copy_data *data)
+static void dump_bb_mem_copy_cmd(int fd, struct xe_mem_copy_data *data)
 {
 	uint32_t *cmd = (uint32_t *) data;
+	uint32_t devid = intel_get_drm_devid(fd);
 
 	igt_info("BB details:\n");
-	igt_info(" dw00: [%08x] <client: 0x%x, opcode: 0x%x, length: %d> "
-		 "[copy type: %d, mode: %d]\n",
-		 cmd[0], data->dw00.client, data->dw00.opcode, data->dw00.length,
-		 data->dw00.copy_type, data->dw00.mode);
-	igt_info(" dw01: [%08x] width: %u\n", cmd[1],
-		 data->dw00.mode == MODE_BYTE ? data->dw01.byte_copy.width :
-						data->dw01.page_copy.width);
+
+	if (intel_graphics_ver(devid) >= IP_VER(20, 0)) {
+		igt_info(" dw00: [%08x] <client: 0x%x, opcode: 0x%x, length: %d> "
+			 "[copy type: %d, mode: %d]\n",
+			 cmd[0], data->dw00.xe2.client, data->dw00.xe2.opcode,
+			 data->dw00.xe2.length, data->dw00.xe2.copy_type,
+			 data->dw00.xe2.mode);
+		igt_info(" dw01: [%08x] width: %u\n", cmd[1],
+			 data->dw00.xe2.mode == MODE_BYTE ? data->dw01.byte_copy.width :
+							data->dw01.page_copy.width);
+	} else {
+		igt_info(" dw00: [%08x] <client: 0x%x, opcode: 0x%x, length: %d> "
+			 "[copy type: %d, mode: %d]\n",
+			 cmd[0], data->dw00.gen12.client,
+			 data->dw00.gen12.opcode, data->dw00.gen12.length,
+			 data->dw00.gen12.copy_type, data->dw00.gen12.mode);
+		igt_info(" dw01: [%08x] width: %u\n", cmd[1],
+			 data->dw00.gen12.mode == MODE_BYTE ? data->dw01.byte_copy.width :
+							data->dw01.page_copy.width);
+	}
 	igt_info(" dw02: [%08x] height: %u\n", cmd[2], data->dw02.height);
 	igt_info(" dw03: [%08x] src pitch: %u\n", cmd[3], data->dw03.src_pitch);
 	igt_info(" dw04: [%08x] dst pitch: %u\n", cmd[4], data->dw04.dst_pitch);
@@ -1910,8 +2006,15 @@ static void dump_bb_mem_copy_cmd(struct xe_mem_copy_data *data)
 		 cmd[7], data->dw07.dst_address_lo);
 	igt_info(" dw08: [%08x] dst offset hi (0x%x)\n",
 		 cmd[8], data->dw08.dst_address_hi);
-	igt_info(" dw09: [%08x] mocs <dst: 0x%x, src: 0x%x>\n",
-		 cmd[8], data->dw09.dst_mocs, data->dw09.src_mocs);
+	if (intel_graphics_ver(devid) >= IP_VER(20, 0)) {
+		igt_info(" dw09: [%08x] mocs <dst: 0x%x, src: 0x%x>\n",
+			 cmd[9], data->dw09.xe2.dst_mocs,
+			 data->dw09.xe2.src_mocs);
+	} else {
+		igt_info(" dw09: [%08x] mocs <dst: 0x%x, src: 0x%x>\n",
+			 cmd[9], data->dw09.gen12.dst_mocs,
+			 data->dw09.gen12.src_mocs);
+	}
 }
 
 static uint64_t emit_blt_mem_copy(int fd, uint64_t ahnd,
@@ -1922,7 +2025,8 @@ static uint64_t emit_blt_mem_copy(int fd, uint64_t ahnd,
 	uint64_t dst_offset, src_offset, shift;
 	uint32_t width, height, width_max, height_max, remain;
 	uint32_t bbe = MI_BATCH_BUFFER_END;
-	uint32_t *bb;
+	uint32_t devid = intel_get_drm_devid(fd);
+	uint8_t *bb;
 
 	if (mem->mode == MODE_BYTE) {
 		data.dw01.byte_copy.width = -1;
@@ -1945,19 +2049,29 @@ static uint64_t emit_blt_mem_copy(int fd, uint64_t ahnd,
 	width = mem->src.width;
 	height = mem->dst.height;
 
-	data.dw00.client = 0x2;
-	data.dw00.opcode = 0x5a;
-	data.dw00.length = 8;
-	data.dw00.mode = mem->mode;
-	data.dw00.copy_type = mem->copy_type;
+	if (intel_graphics_ver(devid) >= IP_VER(20, 0)) {
+		data.dw00.xe2.client = 0x2;
+		data.dw00.xe2.opcode = 0x5a;
+		data.dw00.xe2.length = 8;
+		data.dw00.xe2.mode = mem->mode;
+		data.dw00.xe2.copy_type = mem->copy_type;
+		data.dw09.xe2.src_mocs = mem->src.mocs_index;
+		data.dw09.xe2.dst_mocs = mem->dst.mocs_index;
+	} else {
+		data.dw00.gen12.client = 0x2;
+		data.dw00.gen12.opcode = 0x5a;
+		data.dw00.gen12.length = 8;
+		data.dw00.gen12.mode = mem->mode;
+		data.dw00.gen12.copy_type = mem->copy_type;
+		data.dw09.gen12.src_mocs = mem->src.mocs_index;
+		data.dw09.gen12.dst_mocs = mem->dst.mocs_index;
+	}
 
 	data.dw02.height = height - 1;
 	data.dw05.src_address_lo = src_offset;
 	data.dw06.src_address_hi = src_offset >> 32;
 	data.dw07.dst_address_lo = dst_offset;
 	data.dw08.dst_address_hi = dst_offset >> 32;
-	data.dw09.src_mocs = mem->src.mocs_index;
-	data.dw09.dst_mocs = mem->dst.mocs_index;
 
 	/* For matrix we don't iterate */
 	if (mem->copy_type == TYPE_MATRIX) {
@@ -1983,7 +2097,7 @@ static uint64_t emit_blt_mem_copy(int fd, uint64_t ahnd,
 
 		if (mem->print_bb) {
 			igt_info("[MEM COPY]\n");
-			dump_bb_mem_copy_cmd(&data);
+			dump_bb_mem_copy_cmd(fd, &data);
 		}
 	} else {
 		remain = mem->src.width;
@@ -2017,15 +2131,15 @@ static uint64_t emit_blt_mem_copy(int fd, uint64_t ahnd,
 
 			if (mem->print_bb) {
 				igt_info("[MEM COPY]\n");
-				dump_bb_mem_copy_cmd(&data);
+				dump_bb_mem_copy_cmd(fd, &data);
 			}
 		}
 	}
 
 	if (emit_bbe) {
-		igt_assert(bb_pos + sizeof(uint32_t) < mem->bb.size);
+		igt_assert(bb_pos + sizeof(bbe) < mem->bb.size);
 		memcpy(bb + bb_pos, &bbe, sizeof(bbe));
-		bb_pos += sizeof(uint32_t);
+		bb_pos += sizeof(bbe);
 	}
 
 	munmap(bb, mem->bb.size);
@@ -2117,6 +2231,7 @@ static void emit_blt_mem_set(int fd, uint64_t ahnd,
 	int b;
 	uint32_t *batch;
 	uint32_t value;
+	uint32_t devid = intel_get_drm_devid(fd);
 
 	dst_offset = get_offset_pat_index(ahnd, mem->dst.handle, mem->dst.size,
 					  0, mem->dst.pat_index);
@@ -2131,7 +2246,11 @@ static void emit_blt_mem_set(int fd, uint64_t ahnd,
 	batch[b++] = mem->dst.pitch - 1;
 	batch[b++] = dst_offset;
 	batch[b++] = dst_offset << 32;
-	batch[b++] = value | mem->dst.mocs_index;
+	if (intel_graphics_ver(devid) >= IP_VER(20, 0))
+		batch[b++] = value | (mem->dst.mocs_index << 3);
+	else
+		batch[b++] = value | mem->dst.mocs_index;
+
 	batch[b++] = MI_BATCH_BUFFER_END;
 
 	munmap(batch, mem->bb.size);
