@@ -1023,20 +1023,20 @@ static void smoketest(int i915, const intel_ctx_cfg_t *cfg,
 			    !gem_engine_can_block_ggtt_binder(i915, e)) \
 			igt_dynamic_f("%s", e->name)
 
-igt_main
+int igt_main()
 {
 	const struct intel_execution_engine2 *e;
 	intel_ctx_cfg_t cfg;
 	igt_fd_t(i915);
 
-	igt_fixture {
+	igt_fixture() {
 		i915 = drm_open_driver(DRIVER_INTEL);
 		igt_require_gem(i915);
 		cfg = intel_ctx_cfg_all_physical(i915);
 	}
 
-	igt_subtest_group {
-		igt_fixture {
+	igt_subtest_group() {
+		igt_fixture() {
 			igt_require(gem_has_vm(i915));
 			igt_fork_hang_detector(i915);
 		}
@@ -1071,8 +1071,8 @@ igt_main
 		 * enough to reduce the queue into just another context,
 		 * and so rely on gem_exec_schedule to prove the rest.
 		 */
-		igt_subtest_group {
-			igt_fixture {
+		igt_subtest_group() {
+			igt_fixture() {
 				igt_require(gem_scheduler_enabled(i915));
 				igt_require(gem_scheduler_has_ctx_priority(i915));
 				igt_require(gem_has_vm(i915));
@@ -1100,8 +1100,8 @@ igt_main
 			}
 		}
 
-		igt_subtest_group {
-			igt_fixture {
+		igt_subtest_group() {
+			igt_fixture() {
 				igt_require(gem_scheduler_enabled(i915));
 				igt_require(gem_scheduler_has_ctx_priority(i915));
 				igt_require(gem_has_vm(i915));
@@ -1117,7 +1117,7 @@ igt_main
 			igt_subtest("Q-smoketest-all")
 				smoketest(i915, &cfg, -1, 30);
 
-			igt_fixture {
+			igt_fixture() {
 				intel_allocator_multiprocess_stop();
 			}
 		}
@@ -1129,7 +1129,7 @@ igt_main
 		igt_subtest("exhaust-shared-gtt-lrc")
 			exhaust_shared_gtt(i915, EXHAUST_LRC);
 
-		igt_fixture {
+		igt_fixture() {
 			igt_stop_hang_detector();
 		}
 	}
