@@ -318,11 +318,11 @@ static void single(int i915)
 		munmap(ptr, batch_size);
 }
 
-igt_main
+int igt_main()
 {
 	int i915 = -1;
 
-	igt_fixture {
+	igt_fixture() {
 		i915 = drm_open_driver(DRIVER_INTEL);
 		igt_require_gem(i915);
 
@@ -336,6 +336,6 @@ igt_main
 	igt_subtest("exhaustive")
 		exhaustive(i915);
 
-	igt_fixture
+	igt_fixture()
 		drm_close_driver(i915);
 }
