@@ -5,8 +5,8 @@
 
 #include "config.h"
 
+#include <poll.h>
 #include <pthread.h>
-#include <sys/poll.h>
 #include <sys/ioctl.h>
 #include <sys/mman.h>
 #include <sys/resource.h>
@@ -29,8 +29,11 @@
 /**
  * TEST: gem exec fair
  * Description: Check that GPU time and execution order is fairly distributed across clients
+ * Category: Core
+ * Mega feature: General Core features
+ * Sub-category: CMD submission
+ * Functionality: multi-client systems
  * Feature: cmd_submission
- * Run type: FULL
  *
  * SUBTEST: basic-deadline
  *
@@ -234,7 +237,7 @@ static void delay(int i915,
 		*cs++ = 0;
 	jmp = cs;
 
-	*cs++ = 0x5 << 23; /* MI_ARB_CHECK */
+	*cs++ = MI_ARB_CHECK;
 
 	*cs++ = MI_LOAD_REGISTER_IMM(1);
 	*cs++ = CS_GPR(NOW_TS) + 4;
@@ -576,10 +579,12 @@ static void fair_child(int i915, const intel_ctx_t *ctx,
 
 	/* Synchronize with other children/parent upon construction */
 	if (sv != -1)
-		write(sv, &p_fence, sizeof(p_fence));
+		igt_assert_eq(write(sv, &p_fence, sizeof(p_fence)),
+			      sizeof(p_fence));
 	if (rv != -1)
-		read(rv, &p_fence, sizeof(p_fence));
-	igt_assert(p_fence == -1);
+		igt_assert_eq(read(rv, &p_fence, sizeof(p_fence)),
+			      sizeof(p_fence));
+	igt_assert_eq(p_fence, -1);
 
 	aux_flags = 0;
 	if (intel_gen(intel_get_drm_devid(i915)) < 8)
@@ -847,9 +852,11 @@ static void fairness(int i915, const intel_ctx_cfg_t *cfg,
 		{
 			int sync;
 			for (int child = 0; child < nchild; child++)
-				read(lnk.child[0], &sync, sizeof(sync));
+				igt_assert_eq(read(lnk.child[0], &sync, sizeof(sync)),
+					      sizeof(sync));
 			for (int child = 0; child < nchild; child++)
-				write(lnk.parent[1], &sync, sizeof(sync));
+				igt_assert_eq(write(lnk.parent[1], &sync, sizeof(sync)),
+					      sizeof(sync));
 		}
 
 		while (nfences--)
@@ -1025,9 +1032,9 @@ static void deadline_child(int i915,
 	if (!(flags & DL_PRIO))
 		execbuf.flags |= I915_EXEC_FENCE_IN;
 
-	write(sv, &prev, sizeof(int));
-	read(rv, &prev, sizeof(int));
-	igt_assert(prev == -1);
+	igt_assert_eq(write(sv, &prev, sizeof(int)), sizeof(int));
+	igt_assert_eq(read(rv, &prev, sizeof(int)), sizeof(int));
+	igt_assert_eq(prev, -1);
 
 	prev = execbuf.rsvd2;
 	next = execbuf.rsvd2 >> 32;
@@ -1041,7 +1048,8 @@ static void deadline_child(int i915,
 		gem_execbuf_wr(i915, &execbuf);
 		close(execbuf.rsvd2);
 
-		write(sv, &fence.handle, sizeof(uint32_t));
+		igt_assert_eq(write(sv, &fence.handle, sizeof(uint32_t)),
+			      sizeof(uint32_t));
 
 		prev = next;
 		next = execbuf.rsvd2 >> 32;
@@ -1116,8 +1124,8 @@ static void deadline(int i915, const intel_ctx_cfg_t *cfg,
 		(frame_ns / 1000 / 1000 + 2) * switch_ns + parent_ns;
 	struct intel_execution_engine2 pe = pick_default(i915, cfg);
 	struct intel_execution_engine2 ve = pick_engine(i915, cfg, "vcs0");
-	struct drm_i915_gem_exec_fence *fences = calloc(sizeof(*fences), 32);
-	struct drm_i915_gem_exec_object2 *obj = calloc(sizeof(*obj), 32);
+	struct drm_i915_gem_exec_fence *fences = calloc(32, sizeof(*fences));
+	struct drm_i915_gem_exec_object2 *obj = calloc(32, sizeof(*obj));
 	struct drm_i915_gem_execbuffer2 execbuf = {
 		.buffers_ptr = to_user_pointer(obj),
 		.cliprects_ptr = to_user_pointer(fences),
@@ -1184,10 +1192,12 @@ static void deadline(int i915, const intel_ctx_cfg_t *cfg,
 		}
 
 		for (int i = 0; i < num_children; i++)
-			read(link[i].child[0], &over, sizeof(int));
+			igt_assert_eq(read(link[i].child[0], &over, sizeof(int)),
+				      sizeof(int));
 		igt_info("Testing %d children, with %'dns\n", num_children, child_ns);
 		for (int i = 0; i < num_children; i++)
-			write(link[i].parent[1], &over, sizeof(int));
+			igt_assert_eq(write(link[i].parent[1], &over, sizeof(int)),
+				      sizeof(int));
 
 		over = 0;
 		missed = 0;
@@ -1204,7 +1214,8 @@ static void deadline(int i915, const intel_ctx_cfg_t *cfg,
 
 			sw_sync_timeline_inc(timeline, 1);
 			for (int i = 0; i < num_children; i++) {
-				read(link[i].child[0], &fences[i].handle, sizeof(uint32_t));
+				igt_assert_eq(read(link[i].child[0], &fences[i].handle, sizeof(uint32_t)),
+					      sizeof(uint32_t));
 				fences[i].flags = I915_EXEC_FENCE_WAIT;
 			}
 
@@ -1278,7 +1289,7 @@ static bool set_heartbeat(int i915, const char *name, unsigned int value)
 	return true;
 }
 
-igt_main
+int igt_main()
 {
 	static const struct {
 		const char *name;
@@ -1365,7 +1376,7 @@ igt_main
 	intel_ctx_cfg_t cfg;
 	int i915 = -1;
 
-	igt_fixture {
+	igt_fixture() {
 		igt_require_sw_sync();
 
 		i915 = drm_open_driver_master(DRIVER_INTEL);
@@ -1438,7 +1449,7 @@ igt_main
 		}
 	}
 
-	igt_fixture {
+	igt_fixture() {
 		igt_stop_hang_detector();
 		drm_close_driver(i915);
 	}
