@@ -2074,7 +2074,7 @@ static const char *yesno(bool x)
 	return x ? "yes" : "no";
 }
 
-igt_main
+int igt_main()
 {
 	const struct intel_execution_ring *e;
 	const int ncpus = sysconf(_SC_NPROCESSORS_ONLN);
@@ -2105,7 +2105,7 @@ igt_main
 	unsigned cpu = x86_64_features();
 	int fd = -1;
 
-	igt_fixture {
+	igt_fixture() {
 		igt_require(igt_setup_clflush());
 		fd = drm_open_driver(DRIVER_INTEL);
 		igt_require_gem(fd);
@@ -2123,11 +2123,11 @@ igt_main
 		igt_fork_hang_detector(fd);
 	}
 
-	for (e = intel_execution_rings; e->name; e++) igt_subtest_group {
+	for (e = intel_execution_rings; e->name; e++) igt_subtest_group() {
 		unsigned ring = eb_ring(e);
 		unsigned timeout = 5 + 120*!!e->exec_id;
 
-		igt_fixture {
+		igt_fixture() {
 			gem_require_ring(fd, ring);
 			igt_require(gem_can_store_dword(fd, ring));
 		}
@@ -2208,7 +2208,7 @@ igt_main
 		}
 	}
 
-	igt_fixture {
+	igt_fixture() {
 		igt_stop_hang_detector();
 		drm_close_driver(fd);
 	}
