@@ -185,11 +185,11 @@ test_flink_lifetime(int fd)
 	drm_close_driver(fd2);
 }
 
-igt_main
+int igt_main()
 {
 	igt_fd_t(fd);
 
-	igt_fixture
+	igt_fixture()
 		fd = drm_open_driver(DRIVER_INTEL);
 
 	igt_describe("Check if gem object can be exported to global namespace"
