@@ -716,7 +716,7 @@ static void buf_mode_to_str(uint32_t tiling, bool mixed_tiled,
 		 tiling_str, compression_str[0] ? "-" : "", compression_str);
 }
 
-igt_main_args("dac", NULL, help_str, opt_handler, NULL)
+int igt_main_args("dac", NULL, help_str, opt_handler, NULL)
 {
 	static const struct test_desc {
 		int src_tiling;
@@ -857,12 +857,12 @@ igt_main_args("dac", NULL, help_str, opt_handler, NULL)
 
 	data_t data = {0, };
 
-	igt_fixture {
+	igt_fixture() {
 		data.drm_fd = drm_open_driver_render(DRIVER_INTEL);
 		data.devid = intel_get_drm_devid(data.drm_fd);
 		igt_require_gem(data.drm_fd);
 
-		data.render_copy = igt_get_render_copyfunc(data.devid);
+		data.render_copy = igt_get_render_copyfunc(data.drm_fd);
 		igt_require_f(data.render_copy,
 			      "no render-copy function\n");
 
@@ -942,7 +942,7 @@ igt_main_args("dac", NULL, help_str, opt_handler, NULL)
 		}
 	}
 
-	igt_fixture {
+	igt_fixture() {
 		igt_stop_hang_detector();
 		buf_ops_destroy(data.bops);
 		igt_collection_destroy(set);
