@@ -23,8 +23,14 @@
 #include "xe/xe_query.h"
 #include "xe/xe_util.h"
 
+#define XE_COH_NONE          1
+#define XE_COH_AT_LEAST_1WAY 2
+
 static bool do_slow_check;
 
+static uint32_t create_object(int fd, int r, int size, uint16_t coh_mode,
+			      bool force_cpu_wc);
+
 /**
  * SUBTEST: userptr-coh-none
  * Test category: functionality test
@@ -419,8 +425,30 @@ static void pat_index_render(struct xe_pat_param *p)
 	}
 
 	/* Write some values from the CPU, potentially dirtying the CPU cache */
-	for (i = 0; i < size / sizeof(uint32_t); i++)
-		p->r1_map[i] = i;
+	if (p->r1_compressed) {
+		uint32_t tmp_bo;
+		uint32_t *tmp_map;
+		uint32_t tmp_region = system_memory(fd);
+		uint8_t tmp_pat_index = intel_get_pat_idx_wb(fd);
+
+		tmp_bo = create_object(fd, tmp_region, size,
+				       XE_COH_AT_LEAST_1WAY, false);
+		tmp_map = xe_bo_map(fd, tmp_bo, size);
+
+		for (i = 0; i < size / sizeof(uint32_t); i++)
+			tmp_map[i] = i;
+
+		xe_fast_copy(fd,
+			     tmp_bo, tmp_region, tmp_pat_index,
+			     p->r1_bo, p->r1, p->r1_pat_index,
+			     size);
+
+		munmap(tmp_map, size);
+		gem_close(fd, tmp_bo);
+	} else {
+		for (i = 0; i < size / sizeof(uint32_t); i++)
+			p->r1_map[i] = i;
+	}
 
 	/* And finally ensure we always see the CPU written values */
 	render_copy(ibb,
@@ -725,6 +753,62 @@ static void prime_external_import_coh(void)
 	drm_close_driver(fd2);
 }
 
+static bool has_no_compression_hint(int fd)
+{
+	struct drm_xe_query_config *config = xe_config(fd);
+
+	return config->info[DRM_XE_QUERY_CONFIG_FLAGS] &
+		DRM_XE_QUERY_CONFIG_FLAG_HAS_NO_COMPRESSION_HINT;
+}
+
+/**
+ * SUBTEST: bo-comp-disable-bind
+ * Test category: functionality test
+ * Description: Validates that binding a BO created with
+ * the NO_COMPRESSION flag using a compressed PAT index fails
+ * with -EINVAL on Xe2+ platforms.
+ */
+
+static void bo_comp_disable_bind(int fd)
+{
+	size_t size = xe_get_default_alignment(fd);
+	uint8_t comp_pat_index, uncomp_pat_index;
+	bool supported;
+	uint32_t vm, bo;
+	int ret;
+
+	supported = has_no_compression_hint(fd);
+	ret = __xe_bo_create_caching(fd, 0, size, vram_if_possible(fd, 0),
+				     DRM_XE_GEM_CREATE_FLAG_NO_COMPRESSION,
+				     DRM_XE_GEM_CPU_CACHING_WC, &bo);
+
+	if (!supported) {
+		igt_assert_neq(ret, 0);
+		igt_skip("Missing NO_COMPRESSION support, skipping.\n");
+	}
+
+	igt_assert_eq(ret, 0);
+	vm = xe_vm_create(fd, 0, 0);
+
+	comp_pat_index = intel_get_pat_idx_uc_comp(fd);
+	uncomp_pat_index = intel_get_pat_idx_uc(fd);
+
+	igt_assert_eq(__xe_vm_bind(fd, vm, 0, bo, 0, 0x100000,
+				   size, 0, 0, NULL, 0,
+				   0, comp_pat_index, 0),
+		      -EINVAL);
+
+	igt_assert_eq(__xe_vm_bind(fd, vm, 0, bo, 0, 0x100000,
+				   size, 0, 0, NULL, 0,
+				   0, uncomp_pat_index, 0),
+		      0);
+	xe_vm_unbind_sync(fd, vm, 0, 0x100000, size);
+
+	gem_close(fd, bo);
+
+	xe_vm_destroy(fd, vm);
+}
+
 /**
  * SUBTEST: display-vs-wb-transient
  * Test category: functionality test
@@ -867,9 +951,6 @@ static uint8_t get_pat_idx_wb(int fd, bool *compressed)
 	return intel_get_pat_idx_wb(fd);
 }
 
-#define XE_COH_NONE          1
-#define XE_COH_AT_LEAST_1WAY 2
-
 struct pat_index_entry {
 	uint8_t (*get_pat_index)(int fd, bool *compressed);
 	uint8_t pat_index;
@@ -1130,12 +1211,12 @@ static int opt_handler(int opt, int opt_index, void *data)
 const char *help_str =
 	"  -V\tVerify every dword (might be slow)\n";
 
-igt_main_args("V", NULL, help_str, opt_handler, NULL)
+int igt_main_args("V", NULL, help_str, opt_handler, NULL)
 {
 	uint16_t dev_id;
 	int fd;
 
-	igt_fixture {
+	igt_fixture() {
 		uint32_t seed;
 
 		fd = drm_open_driver(DRIVER_XE);
@@ -1160,6 +1241,9 @@ igt_main_args("V", NULL, help_str, opt_handler, NULL)
 	igt_subtest("prime-external-import-coh")
 		prime_external_import_coh();
 
+	igt_subtest("bo-comp-disable-bind")
+		bo_comp_disable_bind(fd);
+
 	igt_subtest_with_dynamic("pat-index-xelp") {
 		igt_require(intel_graphics_ver(dev_id) <= IP_VER(12, 55));
 		subtest_pat_index_modes_with_regions(fd, xelp_pat_index_modes,
@@ -1193,6 +1277,6 @@ igt_main_args("V", NULL, help_str, opt_handler, NULL)
 	igt_subtest("display-vs-wb-transient")
 		display_vs_wb_transient(fd);
 
-	igt_fixture
+	igt_fixture()
 		drm_close_driver(fd);
 }
