@@ -514,7 +514,7 @@ static void test_llseek_bad(void)
 	drm_close_driver(fd);
 }
 
-igt_main
+int igt_main()
 {
 	struct {
 		const char *name;
@@ -532,7 +532,7 @@ igt_main
 	int i;
 	int fd;
 
-	igt_fixture {
+	igt_fixture() {
 		fd = drm_open_driver(DRIVER_XE);
 	}
 
@@ -541,6 +541,6 @@ igt_main
 			tests[i].fn();
 	}
 
-	igt_fixture
+	igt_fixture()
 		drm_close_driver(fd);
 }
