@@ -238,7 +238,8 @@ exec_queue_reset_wait(int fd)
 	};
 
 	bb_size = xe_bb_size(fd, 0x40000);
-	bo = xe_bo_create(fd, vm, bb_size, vram_if_possible(fd, 0), 0);
+	bo = xe_bo_create(fd, vm, bb_size, vram_if_possible(fd, 0),
+			  DRM_XE_GEM_CREATE_FLAG_NEEDS_VISIBLE_VRAM);
 	data = xe_bo_map(fd, bo, bb_size);
 
 	batch_offset = (char *)&data[0].batch - (char *)data;
@@ -274,11 +275,11 @@ exec_queue_reset_wait(int fd)
 	}
 }
 
-igt_main
+int igt_main()
 {
 	int fd;
 
-	igt_fixture
+	igt_fixture()
 		fd = drm_open_driver(DRIVER_XE);
 
 	igt_subtest("reltime")
@@ -299,6 +300,6 @@ igt_main
 	igt_subtest("exec_queue-reset-wait")
 		exec_queue_reset_wait(fd);
 
-	igt_fixture
+	igt_fixture()
 		drm_close_driver(fd);
 }
