@@ -27,9 +27,7 @@
  * Category: Display
  * Description: Tests behaviour of CRC
  * Driver requirement: i915, xe
- * Functionality: crc
  * Mega feature: General Display Features
- * Test category: functionality test
  */
 
 #include "igt.h"
@@ -58,17 +56,14 @@
  *
  * SUBTEST: suspend-read-crc
  * Description: Suspend test for pipe CRC reads
- * Functionality: crc, suspend
  *
  * SUBTEST: hang-read-crc
  * Description: Hang test for pipe CRC read
- * Functionality: crc, hang
  */
 
 /**
  * SUBTEST: compare-crc-sanitycheck-%s
  * Description: Basic sanity check for CRC mismatches with %arg[1]
- * Functionality: crc, pixel_formats
  *
  * arg[1]:
  *
@@ -373,7 +368,7 @@ static int opt_handler(int opt, int opt_index, void *_data)
 const char *help_str =
 	"  -e \tExtended tests.\n";
 
-igt_main_args("e", NULL, help_str, opt_handler, NULL)
+int igt_main_args("e", NULL, help_str, opt_handler, NULL)
 {
 	enum pipe pipe;
 	igt_output_t *output;
@@ -398,7 +393,7 @@ igt_main_args("e", NULL, help_str, opt_handler, NULL)
 	int i;
 	last_pipe = 0;
 
-	igt_fixture {
+	igt_fixture() {
 		data.drm_fd = drm_open_driver_master(DRIVER_ANY);
 
 		kmstest_set_vt_graphics_mode();
@@ -433,10 +428,16 @@ igt_main_args("e", NULL, help_str, opt_handler, NULL)
 
 				igt_dynamic_f("pipe-%s-%s", kmstest_pipe_name(pipe), output->name) {
 					if (tests[i].flags & TEST_SUSPEND) {
+						enum igt_suspend_test test = SUSPEND_TEST_NONE;
+
 						test_read_crc(&data, pipe, output, 0);
 
+						/* rtcwake cmd is not supported on MTK devices */
+						if (is_mtk_device(data.drm_fd))
+							test = SUSPEND_TEST_DEVICES;
+
 						igt_system_suspend_autoresume(SUSPEND_STATE_MEM,
-									      SUSPEND_TEST_NONE);
+									      test);
 
 						test_read_crc(&data, pipe, output, 0);
 					} else if (tests[i].flags & TEST_HANG) {
@@ -498,7 +499,7 @@ igt_main_args("e", NULL, help_str, opt_handler, NULL)
 		}
 	}
 
-	igt_fixture {
+	igt_fixture() {
 		igt_display_fini(&data.display);
 		drm_close_driver(data.drm_fd);
 	}
