@@ -29,15 +29,23 @@
  * Mega feature: General Display Features
  */
 
+#include <fcntl.h>
+#include <limits.h>
+#include <poll.h>
+#include <sys/ioctl.h>
+#include <time.h>
+
 #include "igt.h"
 #include "igt_device.h"
 #include "igt_debugfs.h"
 #include "igt_sysfs.h"
-#include <fcntl.h>
 
-#include <poll.h>
-#include <sys/ioctl.h>
-#include <time.h>
+#include "lib/intel_blt.h"
+#include "lib/intel_common.h"
+#include "lib/intel_pat.h"
+#include "lib/intel_mocs.h"
+#include "xe/xe_ioctl.h"
+#include "xe/xe_query.h"
 
 /**
  * SUBTEST: D3hot
@@ -128,24 +136,64 @@ static igt_output_t *setup_display(int importer_fd, igt_display_t *display,
 	return output;
 }
 
+static uint32_t *create_and_map_dumb_bo(int fd, struct dumb_bo *bo,
+					uint32_t width, uint32_t height,
+					uint32_t bpp, uint32_t *pitch,
+					uint64_t *size)
+{
+	uint32_t *ptr;
+	const uint32_t alloc_w = ALIGN(width, 256);
+
+	bo->height = height;
+	bo->bpp = bpp;
+	bo->handle = kmstest_dumb_create(fd, alloc_w,
+					 bo->height, bo->bpp, pitch, size);
+
+	ptr = kmstest_dumb_map_buffer(fd, bo->handle, *size, PROT_READ | PROT_WRITE);
+
+	return ptr;
+}
+
+static uint32_t *prepare_xe_dgfx_scratch(int exporter_fd, struct dumb_bo *scratch)
+{
+	uint32_t *ptr;
+	struct blt_copy_data ex_blt = {};
+	struct blt_copy_object *src = NULL;
+
+	igt_debug("Preparing scratch buffer for DGfx exporter\n");
+
+	blt_copy_init(exporter_fd, &ex_blt);
+	src = blt_create_object(&ex_blt, DRM_XE_MEM_REGION_CLASS_VRAM,
+				scratch->width, scratch->height,
+				scratch->bpp, 0, T_LINEAR,
+				COMPRESSION_DISABLED, 0, true);
+	scratch->handle = src->handle;
+	scratch->size = src->size;
+	scratch->pitch = src->pitch;
+	ptr = xe_bo_mmap_ext(exporter_fd, scratch->handle, scratch->size,
+			     PROT_READ | PROT_WRITE);
+
+	return ptr;
+}
+
 static void prepare_scratch(int exporter_fd, struct dumb_bo *scratch,
 			    drmModeModeInfo *mode, uint32_t color)
 {
 	uint32_t *ptr;
+	bool is_dgfx;
 
 	scratch->width = mode->hdisplay;
 	scratch->height = mode->vdisplay;
 	scratch->bpp = 32;
 
-	if (!is_i915_device(exporter_fd)) {
-		scratch->handle = kmstest_dumb_create(exporter_fd,
-						      ALIGN(scratch->width, 256),
-						      scratch->height, scratch->bpp,
-						      &scratch->pitch, &scratch->size);
+	if (drm_get_chipset(exporter_fd) == DRIVER_VGEM)
+		is_dgfx = false;
+	else
+		is_dgfx = is_intel_dgfx(exporter_fd);
 
-		ptr = kmstest_dumb_map_buffer(exporter_fd, scratch->handle,
-					      scratch->size, PROT_WRITE);
-	} else {
+	if (is_xe_device(exporter_fd) && is_dgfx) {
+		ptr = prepare_xe_dgfx_scratch(exporter_fd, scratch);
+	} else if (is_i915_device(exporter_fd)) {
 		struct igt_fb fb;
 
 		igt_init_fb(&fb, exporter_fd, mode->hdisplay, mode->vdisplay,
@@ -165,6 +213,11 @@ static void prepare_scratch(int exporter_fd, struct dumb_bo *scratch,
 
 		ptr = gem_mmap__device_coherent(exporter_fd, scratch->handle, 0, scratch->size,
 						PROT_WRITE | PROT_READ);
+	} else {
+		ptr = create_and_map_dumb_bo(exporter_fd, scratch,
+					     scratch->width, scratch->height,
+					     scratch->bpp, &scratch->pitch,
+					     &scratch->size);
 	}
 
 	for (size_t idx = 0; idx < scratch->size / sizeof(*ptr); ++idx)
@@ -184,40 +237,112 @@ static void prepare_fb(int importer_fd, struct dumb_bo *scratch, struct igt_fb *
 }
 
 static void import_fb(int importer_fd, struct igt_fb *fb,
-		      int dmabuf_fd, uint32_t pitch)
+		      int dmabuf_fd, struct dumb_bo *scratch)
 {
 	uint32_t offsets[4] = {}, pitches[4] = {}, handles[4] = {}, temp_buf_handle;
 	int ret;
 
-	if (is_i915_device(importer_fd)) {
-		if (gem_has_lmem(importer_fd)) {
-			uint64_t ahnd = get_reloc_ahnd(importer_fd, 0);
-			uint64_t fb_size = 0;
+	igt_debug("Importing FB: importer_fd=%d, dmabuf_fd=%d, width=%u, height=%u\n",
+		  importer_fd, dmabuf_fd, fb->width, fb->height);
 
-			igt_info("Importer is dGPU\n");
-			temp_buf_handle = prime_fd_to_handle(importer_fd, dmabuf_fd);
-			igt_assert(temp_buf_handle > 0);
-			fb->gem_handle = igt_create_bo_with_dimensions(importer_fd, fb->width, fb->height,
-								       fb->drm_format, fb->modifier, pitch, &fb_size, NULL, NULL);
-			igt_assert(fb->gem_handle > 0);
+	if (is_intel_dgfx(importer_fd)) {
+		uint64_t fb_size = 0;
+		uint64_t ahnd = 0;
+
+		temp_buf_handle = prime_fd_to_handle(importer_fd, dmabuf_fd);
+		igt_assert(temp_buf_handle > 0);
+		fb->gem_handle = igt_create_bo_with_dimensions(importer_fd, fb->width,
+							       fb->height, fb->drm_format,
+							       fb->modifier, scratch->pitch,
+							       &fb_size, NULL, NULL);
+		igt_assert(fb->gem_handle > 0);
+
+		if (is_i915_device(importer_fd)) {
+			ahnd = get_reloc_ahnd(importer_fd, 0);
 
 			igt_blitter_src_copy(importer_fd, ahnd, 0, NULL, temp_buf_handle,
-					     0, pitch, fb->modifier, 0, 0, fb_size, fb->width,
-					     fb->height, 32, fb->gem_handle, 0, pitch, fb->modifier,
-					     0, 0, fb_size);
+						 0, scratch->pitch, fb->modifier,
+						 0, 0, fb_size, fb->width,
+						 fb->height, 32, fb->gem_handle,
+						 0, scratch->pitch, fb->modifier,
+						 0, 0, fb_size);
 
 			gem_sync(importer_fd, fb->gem_handle);
 			gem_close(importer_fd, temp_buf_handle);
 			put_ahnd(ahnd);
-		} else {
-			fb->gem_handle = prime_fd_to_handle(importer_fd, dmabuf_fd);
+		}  else if (is_xe_device(importer_fd)) {
+			uint32_t xe_bb;
+			struct blt_copy_data blt = {0};
+			struct blt_copy_object *src, *dst;
+			uint32_t vm, xe_exec;
+			intel_ctx_t *xe_ctx;
+			uint64_t bb_size;
+
+			struct drm_xe_engine_class_instance inst = {
+				.engine_class = DRM_XE_ENGINE_CLASS_COPY,
+			};
+
+			vm = xe_vm_create(importer_fd, 0, 0);
+			xe_exec = xe_exec_queue_create(importer_fd, vm, &inst, 0);
+			xe_ctx = intel_ctx_xe(importer_fd, vm, xe_exec, 0, 0, 0);
+			ahnd = intel_allocator_open_full(importer_fd, xe_ctx->vm, 0, 0,
+							 INTEL_ALLOCATOR_SIMPLE,
+							 ALLOC_STRATEGY_LOW_TO_HIGH, 0);
+			bb_size = xe_bb_size(importer_fd, SZ_4K);
+			igt_calc_fb_size(fb);
+
+			fb->gem_handle = xe_bo_create(importer_fd, 0, fb->size,
+						      vram_if_possible(importer_fd, 0),
+						      DRM_XE_GEM_CREATE_FLAG_NEEDS_VISIBLE_VRAM);
+			igt_require(fb->gem_handle);
+
+			blt_copy_init(importer_fd, &blt);
+
+			src = blt_create_object(&blt, vram_if_possible(importer_fd, 0),
+						scratch->width, scratch->height, 32, 0,
+						T_LINEAR, COMPRESSION_DISABLED, 0, true);
+			blt_set_object(src, temp_buf_handle, scratch->size,
+				       vram_if_possible(importer_fd, 0), 0,
+				       DEFAULT_PAT_INDEX,
+				       T_LINEAR, COMPRESSION_DISABLED, 0);
+
+			dst = blt_create_object(&blt, vram_if_possible(importer_fd, 0),
+						scratch->width, scratch->height, 32, 0,
+						T_LINEAR, COMPRESSION_DISABLED, 0, true);
+			blt_set_object(dst, fb->gem_handle, fb->size,
+				       vram_if_possible(importer_fd, 0), 0,
+				       DEFAULT_PAT_INDEX,
+				       T_LINEAR, COMPRESSION_DISABLED, 0);
+
+			blt.color_depth = CD_32bit;
+			blt_set_copy_object(&blt.src, src);
+			blt_set_copy_object(&blt.dst, dst);
+
+			xe_bb = xe_bo_create(importer_fd, 0, bb_size,
+					     vram_if_possible(importer_fd, 0),
+					     DRM_XE_GEM_CREATE_FLAG_NEEDS_VISIBLE_VRAM);
+
+			blt_set_batch(&blt.bb, xe_bb, bb_size, vram_if_possible(importer_fd, 0));
+			blt_fast_copy(importer_fd, xe_ctx, NULL, ahnd, &blt);
+
+			put_offset(ahnd, dst->handle);
+			put_offset(ahnd, src->handle);
+			put_offset(ahnd, xe_bb);
+
+			xe_exec_queue_destroy(importer_fd, xe_exec);
+			xe_vm_destroy(importer_fd, vm);
+			blt_destroy_object(importer_fd, src);
+			gem_close(importer_fd, xe_bb);
+			intel_allocator_close(ahnd);
+			intel_ctx_destroy(importer_fd, xe_ctx);
 		}
 	} else {
 		fb->gem_handle = prime_fd_to_handle(importer_fd, dmabuf_fd);
+		igt_assert(fb->gem_handle > 0);
 	}
 
 	handles[0] = fb->gem_handle;
-	pitches[0] = pitch;
+	pitches[0] = scratch->pitch;
 	offsets[0] = 0;
 
 	ret = drmModeAddFB2(importer_fd, fb->width, fb->height,
@@ -284,7 +409,7 @@ static void test_crc(int exporter_fd, int importer_fd)
 		gem_close(exporter_fd, scratch.handle);
 
 		prepare_fb(importer_fd, &scratch, &fb);
-		import_fb(importer_fd, &fb, dmabuf_fd, scratch.pitch);
+		import_fb(importer_fd, &fb, dmabuf_fd, &scratch);
 		close(dmabuf_fd);
 
 		colors[i].prime_crc.name = "prime";
@@ -333,6 +458,9 @@ static void test_basic_modeset(int drm_fd)
 	enum pipe pipe;
 	drmModeModeInfo *mode;
 	struct igt_fb fb;
+	uint32_t bo;
+	int ret;
+	uint32_t offsets[4] = { 0 };
 
 	igt_device_set_master(drm_fd);
 	igt_display_require(&display, drm_fd);
@@ -341,6 +469,39 @@ static void test_basic_modeset(int drm_fd)
 	mode = igt_output_get_mode(output);
 	igt_assert(mode);
 
+	if (is_xe_device(drm_fd) && xe_has_vram(drm_fd)) {
+		uint32_t strides[4] = { ALIGN(mode->hdisplay * 4, 64) };
+
+		igt_info("Doing modeset on discrete\n");
+
+		igt_init_fb(&fb, drm_fd, mode->hdisplay, mode->vdisplay,
+			    DRM_FORMAT_XRGB8888, DRM_FORMAT_MOD_LINEAR,
+			    IGT_COLOR_YCBCR_BT709, IGT_COLOR_YCBCR_LIMITED_RANGE);
+		igt_calc_fb_size(&fb);
+
+		bo =  xe_bo_create(drm_fd, 0, fb.size, vram_if_possible(drm_fd, 0),
+				   DRM_XE_GEM_CREATE_FLAG_NEEDS_VISIBLE_VRAM);
+		igt_require(bo);
+
+		ret = __kms_addfb(drm_fd, bo,
+				  mode->hdisplay, mode->vdisplay,
+				  DRM_FORMAT_XRGB8888,
+				  DRM_FORMAT_MOD_LINEAR,
+				  strides, offsets, 1,
+				  DRM_MODE_FB_MODIFIERS, &fb.fb_id);
+		igt_assert_eq(ret, 0);
+
+		set_fb(&fb, &display, output);
+		gem_close(drm_fd, bo);
+
+		cairo_surface_destroy(fb.cairo_surface);
+		do_or_die(drmModeRmFB(drm_fd, fb.fb_id));
+
+		igt_display_fini(&display);
+		igt_info("Modeset on discrete done\n");
+		return;
+	}
+
 	igt_create_pattern_fb(drm_fd, mode->hdisplay, mode->vdisplay, DRM_FORMAT_XRGB8888,
 			      DRM_FORMAT_MOD_LINEAR, &fb);
 
@@ -390,14 +551,14 @@ static void kms_poll_disable(void)
 	close(sysfs_fd);
 }
 
-igt_main
+int igt_main()
 {
 	int first_fd = -1;
 	int second_fd_vgem = -1;
 	int second_fd_hybrid = -1;
 	bool first_output, second_output;
 
-	igt_fixture {
+	igt_fixture() {
 		kmstest_set_vt_graphics_mode();
 		/* ANY = anything that is not VGEM */
 		first_fd = __drm_open_driver_another(0, DRIVER_ANY);
@@ -406,8 +567,8 @@ igt_main
 	}
 
 	igt_describe("Hybrid GPU subtests");
-	igt_subtest_group {
-		igt_fixture {
+	igt_subtest_group() {
+		igt_fixture() {
 			second_fd_hybrid = __drm_open_driver_another(1, DRIVER_ANY);
 			igt_require(second_fd_hybrid >= 0);
 			second_output = has_connected_output(second_fd_hybrid);
@@ -460,7 +621,7 @@ igt_main
 			validate_d3_hot(second_fd_hybrid);
 		}
 
-		igt_fixture {
+		igt_fixture() {
 			if (kms_poll_disabled)
 				kms_poll_state_restore();
 
@@ -469,12 +630,14 @@ igt_main
 	}
 
 	igt_describe("VGEM subtests");
-	igt_subtest_group {
-		igt_fixture {
+	igt_subtest_group() {
+		igt_fixture() {
 			second_fd_vgem = __drm_open_driver_another(1, DRIVER_VGEM);
 			igt_require(second_fd_vgem >= 0);
 			if (is_i915_device(first_fd))
 				igt_require(!gem_has_lmem(first_fd));
+			if (is_xe_device(first_fd))
+				igt_require(!xe_has_vram(first_fd));
 		}
 
 		igt_describe("Make a dumb color buffer, export to another device and"
@@ -486,10 +649,10 @@ igt_main
 					test_crc(second_fd_vgem, first_fd);
 		}
 
-		igt_fixture
+		igt_fixture()
 			drm_close_driver(second_fd_vgem);
 	}
 
-	igt_fixture
+	igt_fixture()
 		drm_close_driver(first_fd);
 }
