@@ -28,7 +28,7 @@
 static void
 test_used_bo(int fd, uint64_t timeout)
 {
-	size_t size = PAGE_SIZE;
+	size_t size = VC4_GPU_PAGE_SIZE;
 	uint32_t clearval = 0xaabbccdd + timeout;
 	int handle = igt_vc4_get_cleared_bo(fd, size, clearval);
 	struct drm_vc4_wait_bo wait = {
@@ -57,15 +57,15 @@ test_used_bo(int fd, uint64_t timeout)
 	gem_close(fd, handle);
 }
 
-igt_main
+int igt_main()
 {
 	int fd;
 	int bo_handle;
 
-	igt_fixture {
+	igt_fixture() {
 		fd = drm_open_driver(DRIVER_VC4);
 		igt_require(igt_vc4_is_v3d(fd));
-		bo_handle = igt_vc4_create_bo(fd, PAGE_SIZE);
+		bo_handle = igt_vc4_create_bo(fd, VC4_GPU_PAGE_SIZE);
 	}
 
 	igt_subtest("bad-bo") {
@@ -110,6 +110,6 @@ igt_main
 	igt_subtest("used-bo")
 		test_used_bo(fd, ~0ull);
 
-	igt_fixture
+	igt_fixture()
 		drm_close_driver(fd);
 }
